@@ -157,19 +157,6 @@ var _ = t.Describe("ContainerServer", func() {
 			Expect(err).To(BeNil())
 		})
 
-		It("should succeed load infraContainer", func() {
-			// Given
-			createDummyState()
-			mockDirs(testManifest)
-
-			// When
-			_, err := sut.LoadSandbox(context.Background(), "id")
-
-			// Then
-			Expect(err).To(BeNil())
-			Expect(sut.GetInfraContainer(context.Background(), sandboxID)).NotTo(BeNil())
-		})
-
 		It("should succeed with invalid network namespace", func() {
 			// Given
 			createDummyState()
@@ -421,6 +408,46 @@ var _ = t.Describe("ContainerServer", func() {
 			Expect(sb).To(BeNil())
 			Expect(err).NotTo(BeNil())
 		})
+
+		It("should fail with wrong PodLinuxOverhead", func() {
+			// Given
+			manifest := bytes.Replace(testManifest,
+				[]byte(`"io.kubernetes.cri-o.PodLinuxOverhead": "{}",`),
+				[]byte(`"io.kubernetes.cri-o.PodLinuxOverhead": "wrong",`), 1,
+			)
+			gomock.InOrder(
+				storeMock.EXPECT().
+					FromContainerDirectory(gomock.Any(), gomock.Any()).
+					Return(manifest, nil),
+			)
+
+			// When
+			sb, err := sut.LoadSandbox(context.Background(), "id")
+
+			// Then
+			Expect(sb).To(BeNil())
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should fail with wrong PodLinuxResources", func() {
+			// Given
+			manifest := bytes.Replace(testManifest,
+				[]byte(`"io.kubernetes.cri-o.PodLinuxResources": "{}",`),
+				[]byte(`"io.kubernetes.cri-o.PodLinuxResources": "wrong",`), 1,
+			)
+			gomock.InOrder(
+				storeMock.EXPECT().
+					FromContainerDirectory(gomock.Any(), gomock.Any()).
+					Return(manifest, nil),
+			)
+
+			// When
+			sb, err := sut.LoadSandbox(context.Background(), "id")
+
+			// Then
+			Expect(sb).To(BeNil())
+			Expect(err).NotTo(BeNil())
+		})
 	})
 
 	t.Describe("LoadContainer", func() {
