@@ -8,6 +8,7 @@ import (
 	"os/exec"
 	"path"
 	"path/filepath"
+	"strconv"
 	"strings"
 
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
@@ -22,7 +23,7 @@ import (
 	"github.com/opencontainers/runc/libcontainer/configs"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
 	"k8s.io/apimachinery/pkg/fields"
-	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
+	"k8s.io/utils/cpuset"
 )
 
 const (
@@ -57,15 +58,9 @@ func (h *HighPerformanceHooks) PreStart(ctx context.Context, c *oci.Container, s
 		return nil
 	}
 
-	// creating libctr managers is expensive on v1. Reuse between CPU load balancing and CPU quota
-	podManager, containerManagers, err := libctrManagersForPodAndContainerCgroup(c, s.CgroupParent())
-	if err != nil {
-		return err
-	}
-
 	// disable the CPU load balancing for the container CPUs
 	if shouldCPULoadBalancingBeDisabled(s.Annotations()) {
-		if err := disableCPULoadBalancing(containerManagers); err != nil {
+		if err := disableCPULoadBalancing(c); err != nil {
 			return fmt.Errorf("set CPU load balancing: %w", err)
 		}
 	}
@@ -81,27 +76,23 @@ func (h *HighPerformanceHooks) PreStart(ctx context.Context, c *oci.Container, s
 	// disable the CFS quota for the container CPUs
 	if shouldCPUQuotaBeDisabled(s.Annotations()) {
 		log.Infof(ctx, "Disable cpu cfs quota for container %q", c.ID())
-		if err := setCPUQuota(podManager, containerManagers); err != nil {
+		if err := setCPUQuota(s.CgroupParent(), c); err != nil {
 			return fmt.Errorf("set CPU CFS quota: %w", err)
 		}
 	}
 
 	// Configure c-states for the container CPUs.
 	if configure, value := shouldCStatesBeConfigured(s.Annotations()); configure {
-		log.Infof(ctx, "Configure c-states for container %q to %q", c.ID(), value)
-		switch value {
-		case annotationEnable:
-			// Enable all c-states.
-			if err := setCPUPMQOSResumeLatency(c, "0"); err != nil {
+		maxLatency, err := convertAnnotationToLatency(value)
+		if err != nil {
+			return err
+		}
+
+		if maxLatency != "" {
+			log.Infof(ctx, "Configure c-states for container %q to %q (pm_qos_resume_latency_us: %q)", c.ID(), value, maxLatency)
+			if err := setCPUPMQOSResumeLatency(c, maxLatency); err != nil {
 				return fmt.Errorf("set CPU PM QOS resume latency: %w", err)
 			}
-		case annotationDisable:
-			// Lock the c-state to C0.
-			if err := setCPUPMQOSResumeLatency(c, "n/a"); err != nil {
-				return fmt.Errorf("set CPU PM QOS resume latency: %w", err)
-			}
-		default:
-			return fmt.Errorf("invalid annotation value %s", value)
 		}
 	}
 
@@ -213,19 +204,34 @@ func annotationValueDeprecationWarning(annotation string) string {
 // Since CRI-O is the owner of the container cgroup, it must set this value for
 // the container. Some other entity (kubelet, external service) must ensure this is the case for all
 // other cgroups that intersect (at minimum: all parent cgroups of this cgroup).
-func disableCPULoadBalancing(containerManagers []cgroups.Manager) error {
+func disableCPULoadBalancing(c *oci.Container) error {
+	lspec := c.Spec().Linux
+	if lspec == nil ||
+		lspec.Resources == nil ||
+		lspec.Resources.CPU == nil ||
+		lspec.Resources.CPU.Cpus == "" {
+		return fmt.Errorf("find container %s CPUs", c.ID())
+	}
+
 	if node.CgroupIsV2() {
 		return fmt.Errorf("disabling CPU load balancing on cgroupv2 not yet supported")
 	}
 
-	for i := len(containerManagers) - 1; i >= 0; i-- {
-		cpusetPath := containerManagers[i].Path("cpuset")
-		if err := cgroups.WriteFile(cpusetPath, "cpuset.sched_load_balance", "0"); err != nil {
-			return err
-		}
+	pid, err := c.Pid()
+	if err != nil {
+		return fmt.Errorf("failed to get pid of container %s: %w", c.ID(), err)
+	}
+	controllers, err := cgroups.ParseCgroupFile("/proc/" + strconv.Itoa(pid) + "/cgroup")
+	if err != nil {
+		return fmt.Errorf("failed to get cgroups of container %s: %w", c.ID(), err)
 	}
 
-	return nil
+	cpusetPath, ok := controllers["cpuset"]
+	if !ok {
+		return fmt.Errorf("failed to get cpuset of container %s", c.ID())
+	}
+
+	return cgroups.WriteFile("/sys/fs/cgroup/cpuset"+cpusetPath, "cpuset.sched_load_balance", "0")
 }
 
 func setIRQLoadBalancing(ctx context.Context, c *oci.Container, enable bool, irqSmpAffinityFile, irqBalanceConfigFile string) error {
@@ -277,19 +283,21 @@ func setIRQLoadBalancing(ctx context.Context, c *oci.Container, enable bool, irq
 	return nil
 }
 
-func setCPUQuota(podManager cgroups.Manager, containerManagers []cgroups.Manager) error {
-	if err := disableCPUQuotaForCgroup(podManager); err != nil {
+func setCPUQuota(parentDir string, c *oci.Container) error {
+	containerCgroup, containerCgroupParent, systemd, err := containerCgroupAndParent(parentDir, c)
+	if err != nil {
 		return err
 	}
-	for _, containerManager := range containerManagers {
-		if err := disableCPUQuotaForCgroup(containerManager); err != nil {
-			return err
-		}
+	podCgroup := filepath.Base(containerCgroupParent)
+	podCgroupParent := filepath.Dir(containerCgroupParent)
+
+	if err := disableCPUQuotaForCgroup(podCgroup, podCgroupParent, systemd); err != nil {
+		return err
 	}
-	return nil
+	return disableCPUQuotaForCgroup(containerCgroup, containerCgroupParent, systemd)
 }
 
-func libctrManagersForPodAndContainerCgroup(c *oci.Container, parentDir string) (podManager cgroups.Manager, containerManagers []cgroups.Manager, _ error) {
+func containerCgroupAndParent(parentDir string, c *oci.Container) (ctrCgroup, parentCgroup string, systemd bool, _ error) {
 	var (
 		cgroupManager cgmgr.CgroupManager
 		err           error
@@ -304,60 +312,25 @@ func libctrManagersForPodAndContainerCgroup(c *oci.Container, parentDir string)
 		// Programming error, this is only possible if the manager string is invalid.
 		panic(err)
 	}
-
-	containerCgroupFullPath, err := cgroupManager.ContainerCgroupAbsolutePath(parentDir, c.ID())
+	cgroupPath, err := cgroupManager.ContainerCgroupAbsolutePath(parentDir, c.ID())
 	if err != nil {
-		return nil, nil, err
+		return "", "", false, err
 	}
-
-	podCgroupFullPath := filepath.Dir(containerCgroupFullPath)
-	podManager, err = libctrManager(filepath.Base(podCgroupFullPath), filepath.Dir(podCgroupFullPath), cgroupManager.IsSystemd())
-	if err != nil {
-		return nil, nil, err
-	}
-
-	containerCgroup := filepath.Base(containerCgroupFullPath)
+	containerCgroup := filepath.Base(cgroupPath)
 	// A quirk of libcontainer's cgroup driver.
+	// See explanation in disableCPUQuotaForCgroup function.
 	if cgroupManager.IsSystemd() {
 		containerCgroup = c.ID()
 	}
-
-	containerManager, err := libctrManager(containerCgroup, filepath.Dir(containerCgroupFullPath), cgroupManager.IsSystemd())
-	if err != nil {
-		return nil, nil, err
-	}
-	containerManagers = []cgroups.Manager{containerManager}
-
-	// crun actually does the cgroup configuration in a child of the cgroup CRI-O expects to be the container's
-	extraManager, err := trueContainerCgroupManager(containerCgroupFullPath)
-	if err != nil {
-		return nil, nil, err
-	}
-	if extraManager != nil {
-		containerManagers = append(containerManagers, extraManager)
-	}
-	return podManager, containerManagers, nil
+	return containerCgroup, filepath.Dir(cgroupPath), cgroupManager.IsSystemd(), nil
 }
 
-func trueContainerCgroupManager(expectedContainerCgroup string) (cgroups.Manager, error) {
-	// HACK: There isn't really a better way to check if the actual container cgroup is in a child cgroup of the expected.
-	// We could check /proc/$pid/cgroup, but we need to be able to query this after the container exits and the process is gone.
-	// We know the source of this: crun creates a sub cgroup of the container to do the actual management, to enforce systemd's single
-	// owner rule. Thus, we need to hardcode this check.
-	actualContainerCgroup := filepath.Join(expectedContainerCgroup, "container")
-	cgroupRoot := "/sys/fs/cgroup"
-	// Choose cpuset as the cgroup to check, with little reason.
-	if !node.CgroupIsV2() {
-		cgroupRoot += "/cpuset"
+func disableCPUQuotaForCgroup(cgroup, parent string, systemd bool) error {
+	mgr, err := libctrManager(cgroup, parent, systemd)
+	if err != nil {
+		return err
 	}
-	if _, err := os.Stat(filepath.Join(cgroupRoot, actualContainerCgroup)); err != nil {
-		return nil, nil
-	}
-	// must be crun, make another libctrManager. Regardless of cgroup driver, it will be treated as cgroupfs
-	return libctrManager(filepath.Base(actualContainerCgroup), filepath.Dir(actualContainerCgroup), false)
-}
 
-func disableCPUQuotaForCgroup(mgr cgroups.Manager) error {
 	return mgr.Set(&configs.Resources{
 		SkipDevices: true,
 		CpuQuota:    -1,
@@ -670,3 +643,42 @@ func isCgroupParentBestEffort(s *sandbox.Sandbox) bool {
 func isContainerRequestWholeCPU(cSpec *specs.Spec) bool {
 	return *(cSpec.Linux.Resources.CPU.Shares)%1024 == 0
 }
+
+// convertAnnotationToLatency converts the cpu-c-states.crio.io annotation to a maximum
+// latency value in microseconds.
+//
+// The cpu-c-states.crio.io annotation can be used to control c-states in several ways:
+//
+//	enable: enable all c-states (cpu-c-states.crio.io: "enable")
+//	disable: disable all c-states (cpu-c-states.crio.io: "disable")
+//	max_latency: enable c-states with a maximum latency in microseconds
+//	             (for example,  cpu-c-states.crio.io: "max_latency:10")
+//
+// Examples:
+//
+// cpu-c-states.crio.io: "disable" (disable all c-states)
+// cpu-c-states.crio.io: "enable" (enable all c-states)
+// cpu-c-states.crio.io: "max_latency:10" (use a max latency of 10us)
+func convertAnnotationToLatency(annotation string) (maxLatency string, err error) {
+	//nolint:gocritic // this would not be better as a switch statement
+	if annotation == annotationEnable {
+		// Enable all c-states.
+		return "0", nil
+	} else if annotation == annotationDisable {
+		// Disable all c-states.
+		return "n/a", nil //nolint:goconst // there are not 4 occurrences of this string
+	} else if strings.HasPrefix(annotation, "max_latency:") {
+		// Use the latency provided
+		latency, err := strconv.Atoi(strings.TrimPrefix(annotation, "max_latency:"))
+		if err != nil {
+			return "", err
+		}
+
+		// Latency must be greater than 0
+		if latency > 0 {
+			return strconv.Itoa(latency), nil
+		}
+	}
+
+	return "", fmt.Errorf("invalid annotation value %s", annotation)
+}
