@@ -6,9 +6,9 @@ import (
 	"os"
 
 	"github.com/containers/image/v5/types"
-	"github.com/containers/podman/v3/pkg/rootless"
 	cs "github.com/containers/storage"
 	"github.com/cri-o/cri-o/internal/storage"
+	"github.com/cri-o/cri-o/pkg/config"
 	containerstoragemock "github.com/cri-o/cri-o/test/mocks/containerstorage"
 	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
@@ -55,9 +55,18 @@ var _ = t.Describe("Image", func() {
 		ctx = &types.SystemContext{
 			SystemRegistriesConfPath: t.MustTempFile("registries"),
 		}
+		config := &config.Config{
+			SystemContext: &types.SystemContext{
+				SystemRegistriesConfPath: t.MustTempFile("registries"),
+			},
+			ImageConfig: config.ImageConfig{
+				DefaultTransport:   "docker://",
+				InsecureRegistries: []string{},
+			},
+		}
 
 		sut, err = storage.GetImageService(
-			context.Background(), ctx, storeMock, "docker://", []string{},
+			context.Background(), storeMock, config,
 		)
 		Expect(err).To(BeNil())
 		Expect(sut).NotTo(BeNil())
@@ -82,7 +91,7 @@ var _ = t.Describe("Image", func() {
 			// Given
 			// When
 			imageService, err := storage.GetImageService(
-				context.Background(), nil, storeMock, "", []string{},
+				context.Background(), storeMock, &config.Config{},
 			)
 
 			// Then
@@ -93,31 +102,24 @@ var _ = t.Describe("Image", func() {
 		It("should succeed with custom registries.conf", func() {
 			// Given
 			// When
-			imageService, err := storage.GetImageService(
-				context.Background(),
-				&types.SystemContext{
+			config := &config.Config{
+				SystemContext: &types.SystemContext{
 					SystemRegistriesConfPath: "../../test/registries.conf",
 				},
-				storeMock, "", []string{},
+				ImageConfig: config.ImageConfig{
+					DefaultTransport:   "",
+					InsecureRegistries: []string{},
+				},
+			}
+			imageService, err := storage.GetImageService(
+				context.Background(),
+				storeMock, config,
 			)
 
 			// Then
 			Expect(err).To(BeNil())
 			Expect(imageService).NotTo(BeNil())
 		})
-
-		It("should fail to retrieve an image service without storage", func() {
-			// Given
-			storeOptions, err := cs.DefaultStoreOptions(rootless.IsRootless(), rootless.GetRootlessUID())
-			Expect(err).To(BeNil())
-			storeOptions.GraphRoot = ""
-
-			// When
-			_, err = cs.GetStore(storeOptions)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-		})
 	})
 
 	t.Describe("GetStore", func() {
@@ -294,11 +296,15 @@ var _ = t.Describe("Image", func() {
 				storeMock.EXPECT().Image(gomock.Any()).
 					Return(&cs.Image{ID: "id"}, nil),
 			)
-
+			config := &config.Config{
+				SystemContext: ctx,
+				ImageConfig: config.ImageConfig{
+					DefaultTransport:   "",
+					InsecureRegistries: []string{},
+				},
+			}
 			// Create an empty file for the registries config path
-			sut, err := storage.GetImageService(context.Background(),
-				ctx, storeMock, "", []string{},
-			)
+			sut, err := storage.GetImageService(context.Background(), storeMock, config)
 			Expect(err).To(BeNil())
 			Expect(sut).NotTo(BeNil())
 
@@ -398,7 +404,7 @@ var _ = t.Describe("Image", func() {
 						Names: []string{testNormalizedImageName, "localhost/b:latest", "localhost/c:latest"},
 					}, nil),
 
-				storeMock.EXPECT().SetNames(testSHA256, []string{"localhost/b:latest", "localhost/c:latest"}).
+				storeMock.EXPECT().RemoveNames(testSHA256, []string{"docker.io/library/image:latest", "image"}).
 					Return(t.TestError),
 			)
 
@@ -428,6 +434,18 @@ var _ = t.Describe("Image", func() {
 					}, nil),
 				// buildImageCacheItem
 				mockNewImage(storeMock, testNormalizedImageName, testSHA256),
+				storeMock.EXPECT().Image(testNormalizedImageName).
+					Return(&cs.Image{
+						ID: testSHA256,
+						Names: []string{
+							testNormalizedImageName,
+							"localhost/a@sha256:" + testSHA256,
+							"localhost/b@sha256:" + testSHA256,
+							"localhost/c:latest",
+						},
+					}, nil),
+				storeMock.EXPECT().ImageBigData(testSHA256, gomock.Any()).
+					Return(nil, nil),
 				// makeRepoDigests
 				storeMock.EXPECT().ImageBigDataDigest(testSHA256, gomock.Any()).
 					Return(digest.Digest("a:"+testSHA256), nil),
@@ -507,6 +525,15 @@ var _ = t.Describe("Image", func() {
 				return inOrder(
 					// buildImageCacheItem:
 					mockNewImage(storeMock, testSHA256, testSHA256),
+					storeMock.EXPECT().Image(gomock.Any()).
+						Return(&cs.Image{
+							ID: testSHA256,
+							Names: []string{
+								"localhost/c:latest",
+							},
+						}, nil),
+					storeMock.EXPECT().ImageBigData(testSHA256, gomock.Any()).
+						Return(nil, nil),
 					// makeRepoDigests:
 					storeMock.EXPECT().ImageBigDataDigest(testSHA256, gomock.Any()).
 						Return(digest.Digest(""), nil),
@@ -540,6 +567,17 @@ var _ = t.Describe("Image", func() {
 				mockGetStoreImage(storeMock, testNormalizedImageName, testSHA256),
 				// buildImageCacheItem:
 				mockNewImage(storeMock, testNormalizedImageName, testSHA256),
+
+				storeMock.EXPECT().Image(gomock.Any()).
+					Return(&cs.Image{
+						ID: testSHA256,
+						Names: []string{
+							testNormalizedImageName,
+						},
+					}, nil),
+				storeMock.EXPECT().ImageBigData(testSHA256, gomock.Any()).
+					Return(nil, nil),
+
 				// makeRepoDigests:
 				storeMock.EXPECT().ImageBigDataDigest(testSHA256, gomock.Any()).
 					Return(digest.Digest(""), nil),
@@ -728,4 +766,36 @@ var _ = t.Describe("Image", func() {
 			Expect(res).To(BeNil())
 		})
 	})
+
+	t.Describe("CompileRegexpsForPinnedImages", func() {
+		It("should return regexps for exact patterns", func() {
+			patterns := []string{"quay.io/crio/pause:latest", "docker.io/crio/sandbox:latest", "registry.k8s.io/pause:3.9"}
+			regexps := storage.CompileRegexpsForPinnedImages(patterns)
+			Expect(len(regexps)).To(Equal(len(patterns)))
+			Expect(regexps[0].MatchString("quay.io/crio/pause:latest")).To(BeTrue())
+			Expect(regexps[1].MatchString("docker.io/crio/sandbox:latest")).To(BeTrue())
+			Expect(regexps[2].MatchString("registry.k8s.io/pause:3.9")).To(BeTrue())
+		})
+
+		It("should return regexps for keyword patterns", func() {
+			patterns := []string{"*Fedora*"}
+			regexps := storage.CompileRegexpsForPinnedImages(patterns)
+			Expect(len(regexps)).To(Equal(len(patterns)))
+			Expect(regexps[0].MatchString("quay.io/crio/Fedora34:latest")).To(BeTrue())
+		})
+
+		It("should return regexps for glob patterns", func() {
+			patterns := []string{"quay.io/*", "*Fedora*", "docker.io/*"}
+			regexps := storage.CompileRegexpsForPinnedImages(patterns)
+			Expect(len(regexps)).To(Equal(len(patterns)))
+			Expect(regexps[0].MatchString("quay.io/test/image")).To(BeTrue())
+			Expect(regexps[1].MatchString("gcr.io/CRIO-Fedora34")).To(BeTrue())
+			Expect(regexps[2].MatchString("docker.io/test/image")).To(BeTrue())
+		})
+
+		It("should panic for invalid pattern", func() {
+			patterns := []string{"*"}
+			Expect(func() { storage.CompileRegexpsForPinnedImages(patterns) }).To(Panic())
+		})
+	})
 })
