@@ -2,6 +2,7 @@ package config
 
 import (
 	"bytes"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"net"
@@ -38,10 +39,10 @@ import (
 	"github.com/cri-o/cri-o/utils"
 	"github.com/cri-o/cri-o/utils/cmdrunner"
 	"github.com/cri-o/ocicni/pkg/ocicni"
+	"github.com/opencontainers/runtime-spec/specs-go/features"
 	selinux "github.com/opencontainers/selinux/go-selinux"
 	"github.com/sirupsen/logrus"
-
-	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
+	"k8s.io/utils/cpuset"
 )
 
 // Defaults if none are specified
@@ -169,6 +170,9 @@ type RootConfig struct {
 	// If set to false, one must use the external command `crio wipe` to wipe the containers and images in these situations.
 	// The option InternalWipe is deprecated, and will be removed in a future release.
 	InternalWipe bool `toml:"internal_wipe"`
+
+	// InternalRepair is used to repair the affected images.
+	InternalRepair bool `toml:"internal_repair"`
 }
 
 // GetStore returns the container storage for a given configuration
@@ -218,6 +222,10 @@ type RuntimeHandler struct {
 	// PlatformRuntimePaths defines a configuration option that specifies
 	// the runtime paths for different platforms.
 	PlatformRuntimePaths map[string]string `toml:"platform_runtime_paths,omitempty"`
+
+	// Output of the "features" subcommand.
+	// This is populated dynamically and not read from config.
+	features features.Features
 }
 
 // Multiple runtime Handlers in a map
@@ -453,6 +461,10 @@ type RuntimeConfig struct {
 	// when it is running in the host network namespace
 	// https://github.com/cri-o/cri-o/issues/5501
 	HostNetworkDisableSELinux bool `toml:"hostnetwork_disable_selinux"`
+
+	// Option to disable hostport mapping in CRI-O
+	// Default value is 'false'
+	DisableHostPortMapping bool `toml:"disable_hostport_mapping"`
 }
 
 // ImageConfig represents the "crio.image" TOML config table.
@@ -474,6 +486,11 @@ type ImageConfig struct {
 	// PauseCommand is the path of the binary we run in an infra
 	// container that's been instantiated using PauseImage.
 	PauseCommand string `toml:"pause_command"`
+	// PinnedImages is a list of container images that should be pinned
+	// and not subject to garbage collection by kubelet.
+	// Pinned images will remain in the container runtime's storage until
+	// they are manually removed. Default value: empty list (no images pinned)
+	PinnedImages []string `toml:"pinned_images"`
 	// SignaturePolicyPath is the name of the file which decides what sort
 	// of policy we use when deciding whether or not to trust an image that
 	// we've pulled.  Outside of testing situations, it is strongly advised
@@ -802,6 +819,7 @@ func DefaultConfig() (*Config, error) {
 			VersionFile:       CrioVersionPathTmp,
 			CleanShutdownFile: CrioCleanShutdownFile,
 			InternalWipe:      true,
+			InternalRepair:    false,
 		},
 		APIConfig: APIConfig{
 			Listen:             CrioSocketPath,
@@ -847,6 +865,7 @@ func DefaultConfig() (*Config, error) {
 			rdtConfig:                   rdt.New(),
 			ulimitsConfig:               ulimits.New(),
 			HostNetworkDisableSELinux:   true,
+			DisableHostPortMapping:      false,
 		},
 		ImageConfig: ImageConfig{
 			DefaultTransport:   "docker://",
@@ -1206,10 +1225,27 @@ func (c *RuntimeConfig) ValidateRuntimes() error {
 	for _, invalidHandlerName := range failedValidation {
 		delete(c.Runtimes, invalidHandlerName)
 	}
+	c.initializeRuntimeFeatures()
 
 	return nil
 }
 
+func (c *RuntimeConfig) initializeRuntimeFeatures() {
+	for _, handler := range c.Runtimes {
+		// If this returns an error, we just ignore it and assume the features sub-command is
+		// not supported by the runtime.
+		output, err := cmdrunner.Command(handler.RuntimePath, "features").CombinedOutput()
+		if err != nil {
+			logrus.Errorf("Getting OCI runtime features failed: %s", err)
+			continue
+		}
+		// Ignore errors to Unmarshal too, we can't populate it.
+		if err := json.Unmarshal(output, &handler.features); err != nil {
+			logrus.Errorf("Unmarshalling OCI features failed: %s", err)
+		}
+	}
+}
+
 func (c *RuntimeConfig) TranslateMonitorFields(onExecution bool) error {
 	for name, handler := range c.Runtimes {
 		if handler.RuntimeType == DefaultRuntimeType || handler.RuntimeType == "" {
@@ -1507,6 +1543,18 @@ func (r *RuntimeHandler) ValidateRuntimeAllowedAnnotations() error {
 	return nil
 }
 
+// RuntimeSupportsIDMap returns whether this runtime supports the "runtime features"
+// command, and that the output of that command advertises IDMap mounts as an option
+func (r *RuntimeHandler) RuntimeSupportsIDMap() bool {
+	if r.features.Linux == nil || r.features.Linux.MountExtensions == nil || r.features.Linux.MountExtensions.IDMap == nil {
+		return false
+	}
+	if enabled := r.features.Linux.MountExtensions.IDMap.Enabled; enabled == nil || !*enabled {
+		return false
+	}
+	return true
+}
+
 func validateAllowedAndGenerateDisallowedAnnotations(allowed []string) (disallowed []string, _ error) {
 	disallowedMap := make(map[string]struct{})
 	for _, ann := range annotations.AllAllowedAnnotations {
