@@ -1,19 +1,25 @@
 package config_test
 
 import (
-	"io/ioutil"
 	"os"
+	"os/exec"
 	"path"
 	"path/filepath"
 
 	"github.com/containers/storage"
 	crioann "github.com/cri-o/cri-o/pkg/annotations"
 	"github.com/cri-o/cri-o/pkg/config"
+	"github.com/cri-o/cri-o/utils/cmdrunner"
 
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 )
 
+const (
+	alwaysPresentPath = "/tmp"
+	invalid           = "invalid"
+)
+
 // The actual test suite
 var _ = t.Describe("Config", func() {
 	BeforeEach(beforeEach)
@@ -378,17 +384,6 @@ var _ = t.Describe("Config", func() {
 			Expect(err).NotTo(BeNil())
 		})
 
-		It("should fail on invalid conmon cgroup", func() {
-			// Given
-			sut.ConmonCgroup = "wrong"
-
-			// When
-			err := sut.RuntimeConfig.Validate(nil, false)
-
-			// Then
-			Expect(err).NotTo(BeNil())
-		})
-
 		It("should succeed without defaultRuntime set", func() {
 			// Given
 			sut.DefaultRuntime = ""
@@ -416,7 +411,43 @@ var _ = t.Describe("Config", func() {
 
 		It("should fail on invalid default_sysctls", func() {
 			// Given
-			sut.DefaultSysctls = []string{"invalid"}
+			sut.DefaultSysctls = []string{invalid}
+
+			// When
+			err := sut.RuntimeConfig.Validate(nil, false)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+	})
+	t.Describe("TranslateMonitorFields", func() {
+		It("should fail on invalid conmon cgroup", func() {
+			// Given
+			handler := &config.RuntimeHandler{}
+			sut.ConmonCgroup = "wrong"
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should fail on invalid conmon cgroup", func() {
+			// Given
+			handler := &config.RuntimeHandler{}
+			sut.ConmonCgroup = invalid
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+
+		It("should fail on invalid InfraCtrCPUSet", func() {
+			// Given
+			sut.RuntimeConfig.InfraCtrCPUSet = "unparsable"
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, false)
@@ -425,15 +456,83 @@ var _ = t.Describe("Config", func() {
 			Expect(err).NotTo(BeNil())
 		})
 
-		It("should fail on invalid conmon cgroup", func() {
+		It("should inherit from .Conmon even if bogus", func() {
 			// Given
-			sut.ConmonCgroup = "invalid"
+			sut.Conmon = invalidPath
+			handler := &config.RuntimeHandler{}
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+		It("should inherit from .Conmon", func() {
+			// Given
+			sut.Conmon = validConmonPath()
+			handler := &config.RuntimeHandler{}
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(handler.MonitorPath).To(Equal(sut.Conmon))
+		})
+		It("should inherit from .ConmonEnv", func() {
+			// Given
+			sut.ConmonEnv = []string{"PATH=/usr/bin"}
+			handler := &config.RuntimeHandler{}
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(handler.MonitorEnv).To(Equal(sut.ConmonEnv))
+		})
+		It("should inherit from .ConmonCgroup", func() {
+			// Given
+			sut.ConmonCgroup = "system.slice"
+			handler := &config.RuntimeHandler{}
+
+			// When
+			err := sut.RuntimeConfig.TranslateMonitorFieldsForHandler(handler, true)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(handler.MonitorCgroup).To(Equal(sut.ConmonCgroup))
+		})
+
+		It("should configure a taskset prefix for cmdrunner for a valid InfraCtrCPUSet", func() {
+			executable, err := exec.LookPath("taskset")
+			if err != nil {
+				Skip("this test relies on 'taskset' being present")
+			}
+
+			// Given
+			cmdrunner.ResetPrependedCmd()
+			sut.RuntimeConfig.InfraCtrCPUSet = "0"
+
+			// When
+			err = sut.RuntimeConfig.Validate(nil, false)
+
+			// Then
+			Expect(err).To(BeNil())
+			Expect(cmdrunner.GetPrependedCmd()).To(Equal(executable))
+		})
+
+		It("should not configure a taskset prefix for cmdrunner for an empty InfraCtrCPUSet", func() {
+			// Given
+			cmdrunner.ResetPrependedCmd()
+			sut.RuntimeConfig.InfraCtrCPUSet = ""
 
 			// When
 			err := sut.RuntimeConfig.Validate(nil, false)
 
 			// Then
-			Expect(err).NotTo(BeNil())
+			Expect(err).To(BeNil())
+			Expect(cmdrunner.GetPrependedCmd()).To(Equal(""))
 		})
 	})
 
@@ -543,21 +642,22 @@ var _ = t.Describe("Config", func() {
 		It("should succeed with valid file in $PATH", func() {
 			// Given
 			sut.RuntimeConfig.Conmon = ""
+			handler := &config.RuntimeHandler{MonitorPath: ""}
 
 			// When
-			err := sut.RuntimeConfig.ValidateConmonPath(validConmonPath())
+			err := sut.RuntimeConfig.ValidateConmonPath(validConmonPath(), handler)
 
 			// Then
 			Expect(err).To(BeNil())
-			Expect(sut.RuntimeConfig.Conmon).To(Equal(validConmonPath()))
+			Expect(handler.MonitorPath).To(Equal(validConmonPath()))
 		})
 
 		It("should fail with invalid file in $PATH", func() {
 			// Given
-			sut.RuntimeConfig.Conmon = ""
+			handler := &config.RuntimeHandler{MonitorPath: ""}
 
 			// When
-			err := sut.RuntimeConfig.ValidateConmonPath(invalidPath)
+			err := sut.RuntimeConfig.ValidateConmonPath(invalidPath, handler)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -565,10 +665,10 @@ var _ = t.Describe("Config", func() {
 
 		It("should succeed with valid file outside $PATH", func() {
 			// Given
-			sut.RuntimeConfig.Conmon = validConmonPath()
+			handler := &config.RuntimeHandler{MonitorPath: validConmonPath()}
 
 			// When
-			err := sut.RuntimeConfig.ValidateConmonPath("")
+			err := sut.RuntimeConfig.ValidateConmonPath("", handler)
 
 			// Then
 			Expect(err).To(BeNil())
@@ -576,10 +676,43 @@ var _ = t.Describe("Config", func() {
 
 		It("should fail with invalid file outside $PATH", func() {
 			// Given
-			sut.RuntimeConfig.Conmon = invalidPath
+			handler := &config.RuntimeHandler{MonitorPath: invalidPath}
 
 			// When
-			err := sut.RuntimeConfig.ValidateConmonPath("")
+			err := sut.RuntimeConfig.ValidateConmonPath("", handler)
+
+			// Then
+			Expect(err).NotTo(BeNil())
+		})
+	})
+
+	t.Describe("ValidateImageConfig", func() {
+		It("should succeed with default config", func() {
+			// Given
+			// When
+			err := sut.ImageConfig.Validate(false)
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+
+		It("should succeed on execution and writing permissions", func() {
+			// Given
+			sut.ImageConfig.SignaturePolicyDir = os.TempDir()
+
+			// When
+			err := sut.ImageConfig.Validate(true)
+
+			// Then
+			Expect(err).To(BeNil())
+		})
+
+		It("should fail when SignaturePolicyDir is not absolute", func() {
+			// Given
+			sut.ImageConfig.SignaturePolicyDir = "./wrong/path"
+
+			// When
+			err := sut.ImageConfig.Validate(false)
 
 			// Then
 			Expect(err).NotTo(BeNil())
@@ -751,6 +884,8 @@ var _ = t.Describe("Config", func() {
 			sut.RootConfig.Root = ""
 			sut.RootConfig.Storage = ""
 			sut.RootConfig.StorageOptions = make([]string, 0)
+			// this must be set in case pinns isn't downloaded to the $PATH
+			sut.RuntimeConfig.PinnsPath = alwaysPresentPath
 
 			// When
 			err = sut.Validate(true)
@@ -772,8 +907,10 @@ var _ = t.Describe("Config", func() {
 			defaultStore, err := storage.GetStore(storage.StoreOptions{})
 			Expect(err).To(BeNil())
 
-			sut.RootConfig.RunRoot = "/tmp"
-			sut.RootConfig.Root = "/tmp"
+			sut.RootConfig.RunRoot = alwaysPresentPath
+			sut.RootConfig.Root = alwaysPresentPath
+			// this must be set in case pinns isn't downloaded to the $PATH
+			sut.RuntimeConfig.PinnsPath = alwaysPresentPath
 
 			// When
 			err = sut.Validate(true)
@@ -788,16 +925,14 @@ var _ = t.Describe("Config", func() {
 	t.Describe("ToFile", func() {
 		It("should succeed with default config", func() {
 			// Given
-			tmpfile, err := ioutil.TempFile("", "config")
-			Expect(err).To(BeNil())
-			defer os.Remove(tmpfile.Name())
+			tmpfile := t.MustTempFile("config")
 
 			// When
-			err = sut.ToFile(tmpfile.Name())
+			err := sut.ToFile(tmpfile)
 
 			// Then
 			Expect(err).To(BeNil())
-			_, err = os.Stat(tmpfile.Name())
+			_, err = os.Stat(tmpfile)
 			Expect(err).To(BeNil())
 		})
 
@@ -815,7 +950,7 @@ var _ = t.Describe("Config", func() {
 		It("should succeed with custom config", func() {
 			// Given
 			f := t.MustTempFile("config")
-			Expect(ioutil.WriteFile(f,
+			Expect(os.WriteFile(f,
 				[]byte(`
 					[crio]
 					storage_driver = "overlay2"
@@ -838,7 +973,7 @@ var _ = t.Describe("Config", func() {
 		It("should inherit storage_options from storage.conf and remove duplicates", func() {
 			f := t.MustTempFile("config")
 			// Given
-			Expect(ioutil.WriteFile(f,
+			Expect(os.WriteFile(f,
 				[]byte(`
 					[crio]
 					storage_option = [
@@ -866,10 +1001,100 @@ var _ = t.Describe("Config", func() {
 			}
 		})
 
+		It("should inherit graphroot from storage.conf if crio root is empty", func() {
+			f := t.MustTempFile("config")
+			for _, tc := range []struct {
+				criocfg   []byte
+				graphRoot string
+				expect    string
+			}{
+				{[]byte(`
+				[crio]
+				root = ""
+				`,
+				), "/test/storage", "/test/storage"},
+				{[]byte(`
+				[crio]
+				root = "/test/crio/storage"
+				`,
+				), "/test/storage", "/test/crio/storage"},
+			} {
+				Expect(os.WriteFile(f, tc.criocfg, 0)).To(BeNil())
+				// When
+				defaultcfg := defaultConfig()
+				defaultcfg.Root = tc.graphRoot
+				err := defaultcfg.UpdateFromFile(f)
+
+				// Then
+				Expect(err).To(BeNil())
+				Expect(defaultcfg.Root).To(Equal(tc.expect))
+			}
+		})
+
+		It("should inherit runroot from storage.conf if crio runroot is empty", func() {
+			f := t.MustTempFile("config")
+			for _, tc := range []struct {
+				criocfg []byte
+				runRoot string
+				expect  string
+			}{
+				{[]byte(`
+				[crio]
+				runroot = ""
+				`,
+				), "/test/storage", "/test/storage"},
+				{[]byte(`
+				[crio]
+				runroot = "/test/crio/storage"
+				`,
+				), "/test/storage", "/test/crio/storage"},
+			} {
+				Expect(os.WriteFile(f, tc.criocfg, 0)).To(BeNil())
+				// When
+				defaultcfg := defaultConfig()
+				defaultcfg.RunRoot = tc.runRoot
+				err := defaultcfg.UpdateFromFile(f)
+
+				// Then
+				Expect(err).To(BeNil())
+				Expect(defaultcfg.RunRoot).To(Equal(tc.expect))
+			}
+		})
+
+		It("should inherit runroot from storage.conf if crio runroot is empty", func() {
+			f := t.MustTempFile("config")
+			for _, tc := range []struct {
+				criocfg       []byte
+				storageDriver string
+				expect        string
+			}{
+				{[]byte(`
+				[crio]
+				storage_driver = ""
+				`,
+				), "/test/storage", "/test/storage"},
+				{[]byte(`
+				[crio]
+				storage_driver = "/test/crio/storage"
+				`,
+				), "/test/storage", "/test/crio/storage"},
+			} {
+				Expect(os.WriteFile(f, tc.criocfg, 0)).To(BeNil())
+				// When
+				defaultcfg := defaultConfig()
+				defaultcfg.Storage = tc.storageDriver
+				err := defaultcfg.UpdateFromFile(f)
+
+				// Then
+				Expect(err).To(BeNil())
+				Expect(defaultcfg.Storage).To(Equal(tc.expect))
+			}
+		})
+
 		It("should succeed with custom runtime", func() {
 			// Given
 			f := t.MustTempFile("config")
-			Expect(ioutil.WriteFile(f,
+			Expect(os.WriteFile(f,
 				[]byte("[crio.runtime.runtimes.crun]"), 0),
 			).To(BeNil())
 
@@ -885,7 +1110,7 @@ var _ = t.Describe("Config", func() {
 		It("should succeed with additional runtime", func() {
 			// Given
 			f := t.MustTempFile("config")
-			Expect(ioutil.WriteFile(f,
+			Expect(os.WriteFile(f,
 				[]byte(`
 					[crio.runtime.runtimes.runc]
 					[crio.runtime.runtimes.crun]
@@ -974,12 +1199,12 @@ var _ = t.Describe("Config", func() {
 			Expect(sut.LogLevel).To(Equal("info"))
 
 			configDir := t.MustTempDir("config-dir")
-			Expect(ioutil.WriteFile(
+			Expect(os.WriteFile(
 				filepath.Join(configDir, "00-default"),
 				[]byte("[crio.runtime]\nlog_level = \"debug\"\n"),
 				0o644,
 			)).To(BeNil())
-			Expect(ioutil.WriteFile(
+			Expect(os.WriteFile(
 				filepath.Join(configDir, "01-my-config"),
 				[]byte("[crio.runtime]\nlog_level = \"warning\"\n"),
 				0o644,
@@ -996,7 +1221,7 @@ var _ = t.Describe("Config", func() {
 		It("should fail with invalid config", func() {
 			// Given
 			configDir := t.MustTempDir("config-dir")
-			Expect(ioutil.WriteFile(
+			Expect(os.WriteFile(
 				filepath.Join(configDir, "00-default"),
 				[]byte("[crio.runtime]\nlog_level = true\n"),
 				0o644,
