@@ -237,9 +237,6 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 
 	imageName := imgResult.Name
 	imageRef := imgResult.ID
-	if len(imgResult.RepoDigests) > 0 {
-		imageRef = imgResult.RepoDigests[0]
-	}
 
 	labelOptions, err := ctr.SelinuxLabel(sb.ProcessLabel())
 	if err != nil {
@@ -324,7 +321,8 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 	cgroup2RW := node.CgroupIsV2() && sb.Annotations()[crioann.Cgroup2RWAnnotation] == "true"
 
 	s.resourceStore.SetStageForResource(ctx, ctr.Name(), "container volume configuration")
-	containerVolumes, ociMounts, err := addOCIBindMounts(ctx, ctr, mountLabel, s.config.RuntimeConfig.BindMountPrefix, s.config.AbsentMountSourcesToReject, maybeRelabel, skipRelabel, cgroup2RW, s.Config().Root)
+	idMapSupport := s.Runtime().RuntimeSupportsIDMap(sb.RuntimeHandler())
+	containerVolumes, ociMounts, err := addOCIBindMounts(ctx, ctr, mountLabel, s.config.RuntimeConfig.BindMountPrefix, s.config.AbsentMountSourcesToReject, maybeRelabel, skipRelabel, cgroup2RW, idMapSupport, s.Config().Root)
 	if err != nil {
 		return nil, err
 	}
@@ -625,14 +623,13 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 	created := time.Now()
 	seccompRef := types.SecurityProfile_Unconfined.String()
 	if !ctr.Privileged() {
-		notifier, err := s.config.Seccomp().Setup(
+		notifier, ref, err := s.config.Seccomp().Setup(
 			ctx,
 			s.seccompNotifierChan,
 			containerID,
 			sb.Annotations(),
 			specgen,
 			securityContext.Seccomp,
-			containerConfig.Linux.SecurityContext.SeccompProfilePath,
 		)
 		if err != nil {
 			return nil, fmt.Errorf("setup seccomp: %w", err)
@@ -640,6 +637,7 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 		if notifier != nil {
 			s.seccompNotifiers.Store(containerID, notifier)
 		}
+		seccompRef = ref
 	}
 
 	// Get RDT class
@@ -652,9 +650,8 @@ func (s *Server) createSandboxContainer(ctx context.Context, ctr ctrfactory.Cont
 		// TODO: patch runtime-tools to support setting ClosID via a helper func similar to SetLinuxIntelRdtL3CacheSchema()
 		specgen.Config.Linux.IntelRdt = &rspec.LinuxIntelRdt{ClosID: rdt.ResctrlPrefix + rdtClass}
 	}
-
 	// compute the runtime path for a given container
-	platform := containerInfo.Config.OS + "/" + containerInfo.Config.Architecture
+	platform := containerInfo.Config.Platform.OS + "/" + containerInfo.Config.Platform.Architecture
 	runtimePath, err := s.Runtime().PlatformRuntimePath(sb.RuntimeHandler(), platform)
 	if err != nil {
 		return nil, err
@@ -923,7 +920,7 @@ func clearReadOnly(m *rspec.Mount) {
 	m.Options = append(m.Options, "rw")
 }
 
-func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel, bindMountPrefix string, absentMountSourcesToReject []string, maybeRelabel, skipRelabel, cgroup2RW bool, storageRoot string) ([]oci.ContainerVolume, []rspec.Mount, error) {
+func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel, bindMountPrefix string, absentMountSourcesToReject []string, maybeRelabel, skipRelabel, cgroup2RW, idMapSupport bool, storageRoot string) ([]oci.ContainerVolume, []rspec.Mount, error) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
 
@@ -979,7 +976,7 @@ func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel,
 			log.Warnf(ctx, "Configuration specifies mounting host root to the container root.  This is dangerous (especially with privileged containers) and should be avoided.")
 		}
 
-		if isSubDirectoryOf(storageRoot, m.HostPath) && m.Propagation == types.MountPropagation_PROPAGATION_PRIVATE {
+		if isSubDirectoryOf(storageRoot, m.HostPath) {
 			log.Infof(ctx, "Mount propogration for the host path %s will be set to HostToContainer as it includes the container storage root", m.HostPath)
 			m.Propagation = types.MountPropagation_PROPAGATION_HOST_TO_CONTAINER
 		}
@@ -1068,12 +1065,17 @@ func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel,
 			SelinuxRelabel: m.SelinuxRelabel,
 		})
 
+		uidMappings := getOCIMappings(m.UidMappings)
+		gidMappings := getOCIMappings(m.GidMappings)
+		if (uidMappings != nil || gidMappings != nil) && !idMapSupport {
+			return nil, nil, fmt.Errorf("idmap mounts specified but OCI runtime does not support them. Perhaps the OCI runtime is too old")
+		}
 		ociMounts = append(ociMounts, rspec.Mount{
 			Source:      src,
 			Destination: dest,
 			Options:     options,
-			UIDMappings: getOCIMappings(m.UidMappings),
-			GIDMappings: getOCIMappings(m.GidMappings),
+			UIDMappings: uidMappings,
+			GIDMappings: gidMappings,
 		})
 	}
 
@@ -1097,16 +1099,16 @@ func addOCIBindMounts(ctx context.Context, ctr ctrfactory.Container, mountLabel,
 }
 
 func getOCIMappings(m []*types.IDMapping) []rspec.LinuxIDMapping {
-	if m == nil {
+	if len(m) == 0 {
 		return nil
 	}
 	ids := make([]rspec.LinuxIDMapping, 0, len(m))
-	for i, m := range m {
-		ids[i] = rspec.LinuxIDMapping{
+	for _, m := range m {
+		ids = append(ids, rspec.LinuxIDMapping{
 			ContainerID: m.ContainerId,
 			HostID:      m.HostId,
 			Size:        m.Length,
-		}
+		})
 	}
 	return ids
 }
