@@ -10,6 +10,7 @@ import (
 	"net"
 	"net/http"
 	"os"
+	"os/signal"
 	"path/filepath"
 	"runtime/debug"
 	"strconv"
@@ -33,24 +34,26 @@ import (
 	"github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/resourcestore"
 	"github.com/cri-o/cri-o/internal/runtimehandlerhooks"
+	"github.com/cri-o/cri-o/internal/signals"
 	"github.com/cri-o/cri-o/internal/storage"
 	"github.com/cri-o/cri-o/internal/version"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/server/metrics"
-	"github.com/cri-o/cri-o/server/streaming"
 	"github.com/cri-o/cri-o/utils"
 	"github.com/fsnotify/fsnotify"
 	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
+	"k8s.io/kubelet/pkg/cri/streaming"
 
 	nriIf "github.com/cri-o/cri-o/internal/nri"
 )
 
 const (
-	certRefreshInterval = time.Minute * 5
-	rootlessEnvName     = "_CRIO_ROOTLESS"
+	certRefreshInterval            = time.Minute * 5
+	rootlessEnvName                = "_CRIO_ROOTLESS"
+	irqBalanceConfigRestoreDisable = "disable"
 )
 
 var errSandboxNotCreated = errors.New("sandbox not created")
@@ -88,6 +91,9 @@ type Server struct {
 	seccompNotifierChan chan seccomp.Notification
 	seccompNotifiers    sync.Map
 
+	containerEventClients           sync.Map
+	containerEventStreamBroadcaster sync.Once
+
 	// NRI runtime interface
 	nri *nriAPI
 }
@@ -98,6 +104,7 @@ type pullArguments struct {
 	image         string
 	sandboxCgroup string
 	credentials   imageTypes.DockerAuthConfig
+	namespace     string
 }
 
 // pullOperation is used to synchronize parallel pull operations via the
@@ -253,7 +260,7 @@ func (s *Server) restore(ctx context.Context) []string {
 		}
 	}
 
-	// Go through all the containers and check if it can be restored. If an error occurs, delete the conainer and
+	// Go through all the containers and check if it can be restored. If an error occurs, delete the container and
 	// release the name associated with you.
 	for containerID := range podContainers {
 		err := s.LoadContainer(ctx, containerID)
@@ -419,12 +426,21 @@ func New(
 		return nil, err
 	}
 
-	err = runtimehandlerhooks.RestoreIrqBalanceConfig(config.IrqBalanceConfigFile, runtimehandlerhooks.IrqBannedCPUConfigFile, runtimehandlerhooks.IrqSmpAffinityProcFile)
-	if err != nil {
-		return nil, err
+	if strings.ToLower(strings.TrimSpace(config.IrqBalanceConfigRestoreFile)) != irqBalanceConfigRestoreDisable {
+		log.Infof(ctx, "Attempting to restore irqbalance config from %s", config.IrqBalanceConfigRestoreFile)
+		err = runtimehandlerhooks.RestoreIrqBalanceConfig(context.TODO(), config.IrqBalanceConfigFile, config.IrqBalanceConfigRestoreFile, runtimehandlerhooks.IrqSmpAffinityProcFile)
+		if err != nil {
+			return nil, err
+		}
 	}
 
-	hostportManager := hostport.NewMetaHostportManager()
+	// Check for hostport mapping
+	var hostportManager hostport.HostPortManager
+	if config.RuntimeConfig.DisableHostPortMapping {
+		hostportManager = hostport.NewNoopHostportManager()
+	} else {
+		hostportManager = hostport.NewMetaHostportManager()
+	}
 
 	idMappings, err := getIDMappings(config)
 	if err != nil {
@@ -508,11 +524,12 @@ func New(
 		streamServerConfig.TLSConfig = &tls.Config{
 			GetConfigForClient: certCache.GetConfigForClient,
 			Certificates:       []tls.Certificate{cert},
+			MinVersion:         tls.VersionTLS12,
 		}
 	}
 	s.stream.ctx = ctx
 	s.stream.runtimeServer = s
-	s.stream.streamServer, err = streaming.NewServer(ctx, &streamServerConfig, s.stream)
+	s.stream.streamServer, err = streaming.NewServer(streamServerConfig, s.stream)
 	if err != nil {
 		return nil, fmt.Errorf("unable to create streaming server")
 	}
@@ -527,8 +544,7 @@ func New(
 
 	log.Debugf(ctx, "Sandboxes: %v", s.ContainerServer.ListSandboxes())
 
-	// Start a configuration watcher for the default config
-	s.config.StartWatcher()
+	s.startReloadWatcher(ctx)
 
 	// Start the metrics server if configured to be enabled
 	if s.config.EnableMetrics {
@@ -561,6 +577,26 @@ func New(
 	return s, nil
 }
 
+// startReloadWatcher starts a new SIGHUP go routine.
+func (s *Server) startReloadWatcher(ctx context.Context) {
+	// Setup the signal notifier
+	ch := make(chan os.Signal, 1)
+	signal.Notify(ch, signals.Hup)
+
+	go func() {
+		for {
+			// Block until the signal is received
+			<-ch
+			if err := s.config.Reload(); err != nil {
+				logrus.Errorf("Unable to reload configuration: %v", err)
+				continue
+			}
+		}
+	}()
+
+	log.Infof(ctx, "Registered SIGHUP reload watcher")
+}
+
 func useDefaultUmask() {
 	const defaultUmask = 0o022
 	oldUmask := unix.Umask(defaultUmask)
@@ -1011,6 +1047,7 @@ func (s *Server) generateCRIEvent(ctx context.Context, container *oci.Container,
 		log.Debugf(ctx, "Container event %s generated for %s", eventType, container.ID())
 	default:
 		log.Errorf(ctx, "GenerateCRIEvent: failed to generate event %s for container %s", eventType, container.ID())
+		metrics.Instance().MetricContainersEventsDroppedInc()
 		return
 	}
 }
